package com.springboot.ticketbooking.service;

import java.util.Optional;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import com.springboot.ticketbooking.dao.TicketBookingDao;
import com.springboot.ticketbooking.entities.Ticket;

@Service
public class TicketBookingService {
	
	@Autowired
	private TicketBookingDao ticketBookingDao;
	

	public Ticket createTicket(Ticket ticket) {
				return ticketBookingDao.save(ticket);
	}
	
	@Cacheable(value="ticketsCache",key="#ticketId")
	//This Annotation will keep the data into the TicketsCache
	//Primary key will be the # ticketid which will be map key and data will be as value
	public Optional<Ticket> getTicketById(Integer ticketId) {
				return ticketBookingDao.findById(ticketId);
	}

	@Cacheable(value="ticketsallCache")
	public Iterable<Ticket> getAllBookedTickets() {
		
		return ticketBookingDao.findAll();
	}

	@CacheEvict(value="ticketsCache",key="#ticketId")
	public void deleteBookedTicket(Integer ticketId) {
		
		ticketBookingDao.deleteById(ticketId);
	}

	@CachePut(value="ticketsCache",key="#ticketId")
	//This annotation will update the data with in temporary memory area which is loaded by cache Manager
	//Ideally there is a performance impact because this will udpate the database and update the cache
	//Mostely we cant keep the data which can be changed in the cache memory
		public Ticket updateBookedTicket(Integer ticketId, String newEmail) {
		
		Ticket ticket=ticketBookingDao.findById(ticketId).
				orElse(new Ticket());
		ticket.seteMail(newEmail);
		ticketBookingDao.save(ticket);
		return ticket;
	}

}
